// pic_8259.h

#include "leg.h" 


//purpose:
//   provide control and determine status of 8259 interrupt
//   controller.
//
//   For a description of the 8259 chip itself, @SEE PIC_8259.doc/1.0
//
//Description:
//
//    There is a parent class, PIC_8259, and two derived classes: one for
//    the master chip and one for slave chips.
//    Each physical chip requires its own object.
//
//   Required Parameters for Creation of an object.
//       the 2 hardware addresses to be used for each chip.
//
// Initialization of the 8259.  Each 8259 has to be initialized before other
//    programming of chips can be done.  Initialization of a chip can be
//    performed multiple times.
//    The initialization procedure specifies:
//       - Number of first interrupt vector.  (0..248).
//                                                 @SEE PIC_8259.doc/3.1
//
//       - Triggering mode (level or edge).        @SEE PIC_8259.doc/1.4
//
//       - Use of automatic EOI or software generated EOI
//                                                 @SEE PIC_8259.doc/1.1.3
//
//       - used alone or cascaded                  @SEE PIC_8259.doc/1.1.3
//         if cascaded then
//             - the required arguments depend on whether it is master 
//                   or slave:
//               - if slave then required argument is:
//                   - its slave ID (0..7) (which corresponds to the
//                      interrupt level in the master chip that the
//                      slave is connected to).
//               - If master then required arguments are:
//                   - the list of interrupt levels which are slaves
//                   - and whether Special Fully Nested Mode is required.
//                                                 @SEE PIC_8259.doc/1.3
//
//       - Interrupt number (0..248) of first entry of vector data for chip
//
//       - whether buffered mode                   @SEE PIC_8259.doc/1.9
//
//
//    Following initialization of all 8259's, INITIALIZATION_IS_COMPLETE
//    must be called.
//
//   For a description of initialization,          @SEE PIC_8259.doc/2.1
//**********************************************************************

#if !defined(__PIC_8259_H)
#define __PIC_8259_H

   const MAX_VECTOR_BLOCK = 31;
   // There are 256 interrupt vectors.
   // Interrupt vectors are specified to the 8259 in contiguous "blocks"
   // of 8 so there are 32 blocks (0..31).  When initializing a chip, a
   // value in the range 0..31 is specified to indicate which block will
   // be used for the chip.
   // The number specified (0..31) indicates which vector will be
   // associated with interrupt level 0 of the chip.
   // The other 7 levels for the chip will be associated with the
   // following 7 values.
   //*******************************************************************

class PIC_8259 {
      // This is the seed class for master and slave chips.

protected:

   unsigned char Port_0, Port_1;
   // Every 8259 is addressed by two ports.

   static unsigned char Masters_List_Of_Slaves;
   // List of Levels indicated to master.

   static unsigned char Slaves_List_Of_Master_Level;
   // List of Master levels indicated to slaves on initialization.

   static unsigned char End_Of_Init_Has_Been_Called;
   // Used to determine that calling protocol is being followed.

   static unsigned char Special_Fully_Nested_Mode_In_Use;
   // Indicates how master was initialized

   PIC_8259 (
      unsigned char Interrupt_Control_Port_Addr_0,
      unsigned char Interrupt_Control_Port_Addr_1);

   void Initialize_8259 (
         unsigned char Vector_Block_Value,      // range 0..31
         unsigned char Only_One_Chip                  = FALSE,
         unsigned char Use_Level_Triggered_Mode       = FALSE,
         unsigned char Use_Auto_EOI                   = TRUE,
         unsigned char Bufferred_Mode                 = FALSE);

public :

   void End_Of_All_Initializations ();
      // This must be called after Initialize_8259 has been called for
      // all chips in order for any other method to be used.
      // If any chip is subsequently reinitialized, this must
      // be called again.

   void Disable_Interrupts
      (unsigned char The_Interrupts[],
       int           Number_Of_Interrupts);
        // specified interrupt level(s) will be disabled (others not
        // affected)

   void Enable_Interrupts
      (unsigned char The_Interrupts[],
       int           Number_Of_Interrupts);
         // specified interrupt level(s) will be enabled (others not
         // affected)

   void Set_Priority_To_Be_Lowest
      (unsigned char The_Interrupt);      // range : 0..7
         // By setting this level to be the lowest priority, its
         // successor becomes the highest.  (If 4 is selected to
         // be the lowest, then 5 will have the highest priority.
         // (0 is successor of 7).

   void Issue_Specific_EOI_Command
      (unsigned char The_Interrupt);      // range : 0..7
         // For use in interrupt handler to inform 8259 that the
         // interrupt level can be serviced again by the CPU.
         // Note:
         //    If Special Mask Mode is in use and
         //          the interrupt is disabled and
         //          the interrupt is being serviced (ISR bit is set), then
         //       this command will not clear the bit.

   void Issue_Non_Specific_EOI_Command ();
         // For use in interrupt handler to inform 8259 that the
         // interrupt level can be serviced again by the CPU.
         // The highest priority interrupt in service will be reset.

   void Issue_Specific_EOI_Command_And_Set_Priority_To_Be_Lowest
         (unsigned char The_Interrupt);       // range : 0..7
         // Combines 2 commands: issues EOI and sets priority

   void Issue_Non_Specific_EOI_Command_And_Set_Priority_To_Be_Lowest();
         // Combines 2 commands: issues EOI and sets priority

   void Enable_Automatic_Rotation_For_AEOI ();
         // Force automatic priority rotation (AEOI was specified in
         // initialization)

   void Disable_Automatic_Rotation_For_AEOI ();
         // Stop using automatic priority rotation (AEOI was specified in
         // initialization)

   void Enable_Special_Mask_Mode ();
         // Priorities will not be in effect.  Any interrupt which is not
         // masked may interrupt the one in service.

   void Disable_Special_Mask_Mode ();
         // This returns the 8259 to its default FULLY_NESTED mode.
         // Priorities will be in effect.

   void Poll_8259_And_Determine_Highest_Interrupt_Request 
      (unsigned char &Interrupt_Is_Requested,   // (TRUE/FALSE)
       unsigned char &Interrupt_Level);         // (0..7)
         // If POLL mode is in effect, this function determines
         // whether an Interrupt_Is_Requested (TRUE or FALSE) and
         // if at least one interrupt is requesting service the level
         // of the highest priority interrupt requesting service.
         //  @SEE PIC_8259.doc/3.7

   unsigned char Interrupts_In_Service ();
         // Returns byte representing interrupt levels which are
         // in service.  Range of levels is 0..7
         // Each bit represents a level.
         // Ex. To determine whether level 7 is being serviced:
         // if (Interrupts_In_Service () & 0x80 == 0x80) ...
         // Obtained by reading the ISR

   unsigned char Disabled_Interrupts ();
         // Returns byte mask representing interrupt levels (range is 0..7)
         // which are disabled.  (ie, if level 0 and 1 are disabled, the
         // returned value will be \x03; level 7 disabled -> \x80)
         // obtained by reading the IMR

   unsigned char Interrupts_Requesting_Service ();
         // Returns byte mask representing interrupt levels (range is 0..7)
         // which are requesting service.


};

//----------------------------------------------------------------------
class Master_PIC_8259: public PIC_8259 {

friend Slave_PIC_8259;  // not necessary


public:
   
   Master_PIC_8259 (
      unsigned char Interrupt_Control_Port_Addr_0,
      unsigned char Interrupt_Control_Port_Addr_1) : 
         PIC_8259 (
            Interrupt_Control_Port_Addr_0,
            Interrupt_Control_Port_Addr_1){};

   void Initialize_Master_8259 (
         unsigned char Vector_Block_Value,      // range 0..31
         unsigned char Interrupt_Levels_Which_Are_Slaves [],
         int           Number_Of_Slaves,
         unsigned char Use_Special_Fully_Nested_Mode  = TRUE,
         unsigned char Use_Level_Triggered_Mode       = FALSE,
         unsigned char Use_Auto_EOI                   = TRUE,
         unsigned char Bufferred_Mode                 = FALSE);
      // The default values are not intended to be recommendations.

};

   //*******************************************************************

   // This class is for slave 8259's.

class Slave_PIC_8259: public PIC_8259 {

   unsigned char Masters_Interrupt_Level;

public:
   
   Slave_PIC_8259 (
      unsigned char Interrupt_Control_Port_Addr_0,
      unsigned char Interrupt_Control_Port_Addr_1) : 
         PIC_8259 (
            Interrupt_Control_Port_Addr_0,
            Interrupt_Control_Port_Addr_1){};

   void Initialize_Slave_8259 (
         unsigned char Vector_Block_Value,      // range 0..31
         unsigned char Slave_ID,
         unsigned char Use_Level_Triggered_Mode       = FALSE,
         unsigned char Use_Auto_EOI                   = TRUE,
         unsigned char Bufferred_Mode                 = FALSE);
      // The default values are not intended to be recommendations.

   void Issue_Specific_EOI_Command (unsigned char The_Interrupt);
         // If Special_Fully_Nested_Mode is in effect then
         //    this routine will also issue a SPECIFIC_EOI_COMMAND to
         //    the master if no other interrupts on this chip (the slave)
         //    are requesting service.
         // If Special_Fully_Nested_Mode is not in effect then
         //    a SPECIFIC_EOI_COMMAND will be issued (there cannot be
         //    other interrupts from this chip in service.

   void Issue_Non_Specific_EOI_Command ();
         // If Special_Fully_Nested_Mode is in effect then
         //    this routine will also issue a NON_SPECIFIC_EOI_COMMAND to
         //    the master if no other interrupts on this chip (the slave)
         //    are requesting service.
         // If Special_Fully_Nested_Mode is not in effect then
         //    a SPECIFIC_EOI_COMMAND will be issued (there cannot be
         //    other interrupts from this chip in service.
};
#endif
